"""..."""
from dataclasses import dataclass, field

from aligo.types import DatClass
from aligo.types.Enum import GetFileFields


@dataclass
class GetFileRequest(DatClass):
    """..."""
    file_id: str
    drive_id: str = None
    url_expire_sec: int = field(default=14400, repr=False)
    fields: GetFileFields = field(default='*', repr=False)
    image_thumbnail_process: str = field(default='image/resize,w_160/format,jpeg', repr=False)
    image_url_process: str = field(default='image/resize,w_1920/format,jpeg', repr=False)
    video_thumbnail_process: str = field(default='video/snapshot,t_0,f_jpg,ar_auto,w_800', repr=False)



"""..."""
from aligo.core.Config import V2_TEMPLATE_TEST
from aligo.request import TemplateRequest
from aligo.response import TemplateResponse
from .BaseAligo import BaseAligo


class Template(BaseAligo):
    """..."""

    def _core_template(self, body: TemplateRequest) -> TemplateResponse:
        """模板"""
        response = self.post(V2_TEMPLATE_TEST, body=body)
        return self._result(response, TemplateResponse, status_code=202)



"""..."""
from dataclasses import dataclass

from aligo.types import DatClass, PersonalRightsInfo, PersonalSpaceInfo


@dataclass
class GetPersonalInfoResponse(DatClass):
    """..."""
    personal_rights_info: PersonalRightsInfo = None
    personal_space_info: PersonalSpaceInfo = None



"""..."""
from dataclasses import dataclass

from .Type import DatClass


@dataclass
class AudioMeta(DatClass):
    """..."""
    bitrate: int = None
    duration: int = None
    sample_rate: int = None
    channels: int = None



"""收藏相关"""
from typing import List

from aligo.core import Core
from aligo.request import StarredFileRequest, BatchStarFilesRequest, GetStarredListRequest
from aligo.response import BatchSubResponse
from aligo.types import BaseFile


class Star(Core):
    """..."""
    _MAX_STAR_COUNT: int = 500

    def starred_file(self,
                     file_id: str,
                     starred: bool = True,
                     drive_id: str = None) -> BaseFile:
        """
        官方：收藏(或取消) 文件
        :param file_id: [必须] 文件ID
        :param starred: [必须] 是否收藏
        :param drive_id: [可选] 文件所在的网盘ID
        :return: [BaseFile] 文件信息

        用法示例：
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> star_file = ali.starred_file('file_id')
        >>> print(star_file.name)
        """
        body = StarredFileRequest(
            file_id=file_id,
            starred=starred,
            drive_id=drive_id,
        )
        return self._core_starred_file(body)

    def batch_star_files(self,
                         file_id_list: List[str],
                         starred: bool = True,
                         drive_id: str = None) -> List[BatchSubResponse[BaseFile]]:
        """
        官方：批量收藏(或取消) 文件
        :param file_id_list: [必须] 文件ID列表
        :param starred: [必须] 是否收藏
        :param drive_id: [可选] 文件所在的网盘ID
        :return: [BatchSubResponse] 批量操作结果

        用法示例：
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> star_files = ali.batch_star_files(['file_id1', 'file_id2'])
        >>> print(star_files[0].body.name)
        """
        body = BatchStarFilesRequest(
            drive_id=drive_id,
            file_id_list=file_id_list,
            starred=starred,
        )
        result = self._core_batch_star_files(body)
        return list(result)

    def get_starred_list(self, body: GetStarredListRequest = None, **kwargs) -> List[BaseFile]:
        """
        官方：获取收藏列表
        :param body: [可选] 请求体
        :param kwargs: [可选] 其他参数
        :return: [List[BaseFile]] 收藏列表

        用法示例：
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> starred_list = ali.get_starred_list()
        >>> print(starred_list[0].name)
        """
        if body is None:
            body = GetStarredListRequest(**kwargs)
        result = self._core_get_starred_list(body)
        return list(result)



"""..."""
from dataclasses import dataclass

from aligo.types import DatClass


@dataclass
class AimSearchRequest(DatClass):
    """..."""
    query: str
    marker: str = ''
    return_total_count: bool = True
    order_by: str = 'image_time DESC,last_access_at DESC,updated_at DESC'
    limit: int = 100
    drive_id: str = None



"""..."""
from aligo.core import Core
from aligo.request import GetAudioPlayInfoRequest
from aligo.response import GetAudioPlayInfoResponse


class Audio(Core):
    """音频相关"""

    def get_audio_play_info(self, file_id: str, drive_id: str = None) -> GetAudioPlayInfoResponse:
        """
        官方：获取音频播放信息
        :param file_id: [str] 音频 file_id
        :param drive_id: Optional[str] 音频文件的 drive_id
        :return: [GetAudioPlayInfoResponse]

        用法示例：
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> audio = ali.get_audio_play_info('<file_id>')
        >>> print(audio)
        """
        body = GetAudioPlayInfoRequest(file_id=file_id, drive_id=drive_id)
        return self._core_get_audio_play_info(body)



"""..."""
from typing import List, overload

from aligo.core import Core
from aligo.request import (RestoreFileRequest, GetRecycleBinListRequest, MoveFileToTrashRequest,
                           BatchMoveToTrashRequest, BatchRestoreRequest)
from aligo.response import MoveFileToTrashResponse, BatchSubResponse, RestoreFileResponse
from aligo.types import BaseFile


class Recyclebin(Core):
    """删除文件太过危险, 只提供移动文件到回收站的功能"""

    def move_file_to_trash(self, file_id: str, drive_id: str = None) -> MoveFileToTrashResponse:
        """
        移动文件到回收站
        :param file_id: [必须] 文件ID
        :param drive_id: [可选] 文件所在的网盘ID
        :return: [MoveFileToTrashResponse]

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.move_file_to_trash('<file_id>')
        >>> print(result)
        """
        body = MoveFileToTrashRequest(file_id=file_id, drive_id=drive_id)
        return self._core_move_file_to_trash(body)

    def batch_move_to_trash(self, file_id_list: List[str], drive_id: str = None) -> List[BatchSubResponse]:
        """
        批量移动文件到回收站
        :param file_id_list: [必须] 文件ID列表
        :param drive_id: [可选] 文件所在的网盘ID
        :return: [BatchSubResponse]

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.batch_move_to_trash(['<file1_id>', '<file2_id>'])
        >>> print(result)
        """
        body = BatchMoveToTrashRequest(drive_id=drive_id, file_id_list=file_id_list)
        result = self._core_batch_move_to_trash(body)
        return list(result)

    def restore_file(self, file_id: str, drive_id: str = None) -> RestoreFileResponse:
        """
        恢复文件
        :param file_id: [必须] 文件ID
        :param drive_id: [可选] 文件所在的网盘ID
        :return: [RestoreFileResponse]

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.restore_file('<file_id>')
        >>> print(result)
        """
        body = RestoreFileRequest(drive_id=drive_id, file_id=file_id)
        return self._core_restore_file(body)

    def batch_restore_files(self, file_id_list: List[str], drive_id: str = None) -> List[BatchSubResponse]:
        """
        批量恢复文件
        :param file_id_list: [必须] 文件ID列表
        :param drive_id: [可选] 文件所在的网盘ID
        :return: [BatchSubResponse]

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.batch_restore_files(['<file1_id>', '<file2_id>'])
        >>> print(result)
        """
        body = BatchRestoreRequest(drive_id=drive_id, file_id_list=file_id_list)
        result = self._core_batch_restore_files(body)
        return list(result)

    @overload
    def get_recyclebin_list(self, **kwargs) -> List[BaseFile]:
        """
        获取回收站列表
        :param kwargs: [可选] 其他参数
        :return: [BaseFile]

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.get_recyclebin_list()
        >>> print(result)
        """

    @overload
    def get_recyclebin_list(self, body: GetRecycleBinListRequest) -> List[BaseFile]:
        """
        获取回收站列表
        :param body: [必须] 请求参数
        :return: [BaseFile]

        用法示例:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> body = GetRecycleBinListRequest()
        >>> result = ali.get_recyclebin_list(body)
        >>> print(result)
        """

    def get_recyclebin_list(self, body: GetRecycleBinListRequest = None, **kwargs) -> List[BaseFile]:
        """get_recyclebin_list"""
        if body is None:
            body = GetRecycleBinListRequest(**kwargs)
        result = self._core_get_recyclebin_list(body)
        return list(result)



"""..."""
from typing import Iterator

from aligo.core import BaseAligo
from aligo.core.Config import ADRIVE_V3_FILE_LIST, V2_FILE_WALK, V2_FILE_SCAN
from aligo.request import BatchGetFileRequest, GetFileListRequest, GetFileRequest, BatchRequest, BatchSubRequest
from aligo.response import BatchSubResponse, GetFileListResponse
from aligo.types import BaseFile
from aligo.types.Enum import BaseFileType, BaseFileCategory


class File(BaseAligo):
    """..."""

    def _core_get_file_list(self, body: GetFileListRequest) -> Iterator[BaseFile]:
        """..."""
        yield from self._list_file(ADRIVE_V3_FILE_LIST, body, GetFileListResponse, params={
            'jsonmask': ('next_marker,items(name,file_id,drive_id,type,size,created_at,updated_at,'
                         'category,file_extension,parent_file_id,mime_type,starred,thumbnail,url,'
                         'streams_info,content_hash,user_tags,user_meta,trashed,video_media_metadata,'
                         'video_preview_metadata,sync_meta,sync_device_flag,sync_flag,punish_flag')
        })

    def _core_batch_get_files(self, body: BatchGetFileRequest) -> Iterator[BatchSubResponse]:
        """batch_get_files"""
        if body.drive_id is None:
            body.drive_id = self.default_drive_id

        yield from self.batch_request(BatchRequest(
            requests=[BatchSubRequest(
                id=file_id,
                url='/file/get',
                body=GetFileRequest(
                    drive_id=body.drive_id, file_id=file_id
                )
            ) for file_id in body.file_id_list]
        ), GetFileRequest)

    def _core_walk_file(
            self,
            parent_file_id: str = 'root',
            drive_id: str = None,
            type_: BaseFileType = None,
            url_expire_sec: int = 86400,
            limit: int = 1000,
    ) -> Iterator[BaseFile]:
        """..."""
        yield from self._list_file(V2_FILE_WALK, {
            'parent_file_id': parent_file_id,
            'drive_id': drive_id or self.default_drive_id,
            'type': type_,
            'url_expire_sec': url_expire_sec,
            'limit': limit,
        }, GetFileListResponse)

    def _core_scan_file(
            self,
            drive_id: str = None,
            category: BaseFileCategory = None,
            limit: int = 1000,
    ) -> Iterator[BaseFile]:
        """..."""
        yield from self._list_file(V2_FILE_SCAN, {
            'drive_id': drive_id or self.default_drive_id,
            'category': category,
            'limit': limit,
        }, GetFileListResponse)



"""..."""
from dataclasses import dataclass

from .Type import DatClass
from .Enum import MediaTranscodeStatus


@dataclass
class AudioTranscodeTemplate(DatClass):
    """..."""
    template_id: str = None
    status: MediaTranscodeStatus = None
    url: str = None



"""批量响应"""
from dataclasses import dataclass
from typing import Generic

from aligo.types import DatClass, DataType


@dataclass
class BatchSubResponse(DatClass, Generic[DataType]):
    """..."""
    id: str = None
    status: int = None
    body: DataType = None
    method: str = None



"""..."""
from dataclasses import dataclass, field
from typing import Dict, List

from aligo.core import BaseAligo
from aligo.core.Config import (ADRIVE_V1_USER_CONFIG_GET, MEMBER_HOST, V1_USERS_REWARDS, V2_USER_GET,
                               BUSINESS_V1_USERS_VIP_INFO, USERS_V2_USERS_DEVICE_LIST, USER_HOST)
from aligo.response import RewardSpaceResponse, UsersVipInfoResponse
from aligo.types import BaseUser, UserConfig, LoginDevice
from aligo.types.Type import DatClass


@dataclass
class PhoneBackUp(DatClass):
    folder_id: str = None
    photo_folder_id: str = None
    sub_folder: Dict = None
    video_folder_id: str = None


@dataclass
class BackUpConfig(DatClass):
    # noinspection NonAsciiCharacters
    手机备份: PhoneBackUp = None


@dataclass
class UserData(DatClass):
    back_up_config: BackUpConfig = None


@dataclass
class UserResponse(DatClass):
    avatar: str = field(default=None, repr=False)
    backup_drive_id: str = field(default=None, repr=False)
    created_at: int = field(default=None, repr=False)
    creator: str = field(default=None, repr=False)
    creator_level: str = field(default=None, repr=False)
    default_drive_id: str = None
    default_location: str = field(default=None, repr=False)
    deny_change_password_by_self: bool = field(default=None, repr=False)
    description: str = field(default=None, repr=False)
    domain_id: str = None
    email: str = field(default=None, repr=False)
    expire_at: str = field(default=None, repr=False)
    last_login_time: int = field(default=None, repr=False)
    need_change_password_next_login: bool = field(default=None, repr=False)
    nick_name: str = None
    phone: str = field(default=None, repr=False)
    phone_region: str = field(default=None, repr=False)
    punish_flag: str = field(default=None, repr=False)
    punishments: str = field(default=None, repr=False)
    resource_drive_id: str = None
    role: str = None
    sbox_drive_id: str = field(default=None, repr=False)
    status: str = field(default=None, repr=False)
    updated_at: int = field(default=None, repr=False)
    user_data: UserData = field(default=None, repr=False)
    user_id: str = None
    user_name: str = None
    vip_identity: str = field(default=None, repr=False)


class User(BaseAligo):
    """..."""

    def get_user(self, f5: bool = False) -> BaseUser:
        """
        获取用户信息
        :param f5: [可选] 是否强制刷新
        :return: [BaseUser]
        """
        if self._user is None or f5:
            response = self.post(V2_USER_GET)
            # response.status_code == 200 or self._error_log_exit(response)
            # self._user = BaseUser(**response.json())
            self._user = self._result(response, BaseUser)
        return self._user

    def rewards_space(self, code: str):
        """
        获取奖励空间
        :param code: [必选] 奖励码
        :return: [RewardSpaceResponse]

        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.rewards_space('12345678')
        >>> print(result)
        """
        response = self.post(V1_USERS_REWARDS, MEMBER_HOST, body={
            'code': code
        })
        return self._result(response, RewardSpaceResponse)

    def get_user_config(self) -> UserConfig:
        """获取用户配置信息"""
        response = self.post(ADRIVE_V1_USER_CONFIG_GET, body={})
        return self._result(response, UserConfig)

    def get_vip_info(self) -> UsersVipInfoResponse:
        """获取用户vip信息"""
        response = self.post(BUSINESS_V1_USERS_VIP_INFO, body={})
        return self._result(response, UsersVipInfoResponse)

    def list_login_device(self) -> List[LoginDevice]:
        response = self.post(USERS_V2_USERS_DEVICE_LIST)
        return self._result(response, LoginDevice, field='result.devices')

    def v2_user_get(self) -> UserResponse:
        response = self.post(V2_USER_GET, host=USER_HOST)
        return self._result(response, UserResponse)



"""批量响应"""
from dataclasses import dataclass

from aligo.types import DatClass


@dataclass
class BatchShareFileSaveToDriveResponse(DatClass):
    """..."""
    file_id: str = None
    drive_id: str = None
    domain_id: str = None
    async_task_id: str = None



"""..."""
from typing import List, overload

from aligo.core import Core
from aligo.request import CopyFileRequest, BatchCopyFilesRequest
from aligo.response import CopyFileResponse, BatchSubResponse


class Copy(Core):
    """复制文件相关
    说明：官方虽然提供文件（夹）复制接口，但并未公开文件（夹）复制的功能。
        所以后期可能会失效，不过有替代方案，如果此接口失效我会补上。
    """

    @overload
    def copy_file(self, file_id: str,
                  to_parent_file_id: str = 'root',
                  new_name: str = None,
                  drive_id: str = None,
                  to_drive_id: str = None,
                  **kwargs) -> CopyFileResponse:
        """
        复制文件（夹）
        :param file_id: [str] 文件（夹）ID
        :param to_parent_file_id: Optional[str] 目标目录ID
        :param new_name: Optional[str] 新文件（夹）名
        :param drive_id: Optional[str] 文件（夹）所在的网盘ID
        :param to_drive_id: Optional[str] 目标网盘ID
        :param kwargs: 额外参数
        :return: [CopyFileResponse]

        用法示例：
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.copy_file('<file_id>')
        >>> print(result)
        """

    @overload
    def copy_file(self, body: CopyFileRequest) -> CopyFileResponse:
        """
        复制文件（夹）
        :param body: [CopyFileRequest]
        :return: [CopyFileResponse]

        用法示例：
        >>> from aligo import Aligo, CopyFileRequest
        >>> ali = Aligo()
        >>> result = ali.copy_file(body=CopyFileRequest(file_id='<file_id>'))
        >>> print(result)
        """

    def copy_file(self, file_id: str = None,
                  to_parent_file_id: str = 'root',
                  new_name: str = None,
                  drive_id: str = None,
                  to_drive_id: str = None,
                  body: CopyFileRequest = None,
                  **kwargs) -> CopyFileResponse:
        """copy_file"""
        if body is None:
            body = CopyFileRequest(
                file_id=file_id,
                drive_id=drive_id,
                to_drive_id=to_drive_id,
                to_parent_file_id=to_parent_file_id,
                new_name=new_name,
                **kwargs
            )
        return self._core_copy_file(body)

    @overload
    def batch_copy_files(self,
                         file_id_list: List[str],
                         to_parent_file_id: str = 'root',
                         drive_id: str = None,
                         **kwargs) -> List[BatchSubResponse[CopyFileResponse]]:
        """
        复制文件（夹）
        :param file_id_list: [List[str]] 文件（夹）ID列表
        :param to_parent_file_id: Optional[str] 目标目录ID
        :param drive_id: Optional[str] 文件（夹）所在的网盘ID
        :param kwargs: 额外参数
        :return: [BatchSubResponse]

        用法示例：
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> result = ali.batch_copy_files(file_id_list=['<file1_id>', '<file2_id>'])
        >>> print(result[0].body.file_id)
        """

    @overload
    def batch_copy_files(self, body: BatchCopyFilesRequest) -> List[BatchSubResponse[CopyFileResponse]]:
        """
        复制文件（夹）
        :param body: [BatchCopyFilesRequest]
        :return: [BatchSubResponse]

        用法示例：
        >>> from aligo import Aligo, BatchCopyFilesRequest
        >>> ali = Aligo()
        >>> result = ali.batch_copy_files(body=BatchCopyFilesRequest(file_id_list=['<file1_id>', '<file2_id>']))
        >>> print(result[0].body.file_id)
        """

    def batch_copy_files(self,
                         file_id_list: List[str] = None,
                         to_parent_file_id: str = 'root',
                         drive_id: str = None,
                         body: BatchCopyFilesRequest = None,
                         **kwargs) -> List[BatchSubResponse[CopyFileResponse]]:
        """batch_copy_files"""
        if body is None:
            body = BatchCopyFilesRequest(drive_id=drive_id,
                                         file_id_list=file_id_list,
                                         to_parent_file_id=to_parent_file_id,
                                         **kwargs)
        result = self._core_batch_copy_files(body)
        return list(result)



"""..."""
from typing import List

from aligo.core import Core
from aligo.request import RenameFileRequest, BatchRequest, BatchSubRequest
from aligo.types import BaseFile
from aligo.types.Enum import CheckNameMode


class Update(Core):
    """..."""

    def rename_file(self,
                    file_id: str,
                    name: str,
                    check_name_mode: CheckNameMode = 'refuse',
                    drive_id: str = None) -> BaseFile:
        """
        文件重命名
        :param file_id: [必选] 文件id
        :param name: [必选] 新的文件名
        :param check_name_mode: [可选] 检查文件名模式
        :param drive_id: [可选] 文件所在的网盘id
        :return: [BaseFile] 文件信息

        :Example:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> new_file = ali.rename_file('<file_id>', 'new_name')
        >>> print(new_file.name)
        """
        body = RenameFileRequest(
            name=name,
            file_id=file_id,
            check_name_mode=check_name_mode,
            drive_id=drive_id,
        )
        return self._core_rename_file(body)

    def batch_rename_files(
            self,
            file_id_list: List[str],
            new_name_list: List[str],
            check_name_mode: CheckNameMode = 'refuse',
            drive_id: str = None) -> List[BaseFile]:
        """
        批量重命名文件
        :param file_id_list: [必选] 文件id列表
        :param new_name_list: [必选] 新的文件名列表
        :param check_name_mode: [可选] 检查文件名模式
        :param drive_id: [可选] 文件所在的网盘id
        :return: [List[BaseFile]] 文件信息列表

        :Example:
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> new_file_list = ali.batch_rename_files(['<file_id_list>'], ['<new_name_list>'])
        >>> print(new_file_list[0].name)
        """
        if drive_id is None:
            drive_id = self.default_drive_id

        ids_len = len(file_id_list)
        if ids_len != len(new_name_list):
            self._auth.log.warning(f'长度不一致 file_id_list{ids_len}, new_name_list {len(new_name_list)}')
            return []

        result = []
        for i in self.batch_request(BatchRequest(
                requests=[BatchSubRequest(
                    id=file_id_list[j],
                    url='/file/update',
                    body=RenameFileRequest(
                        name=new_name_list[j],
                        file_id=file_id_list[j],
                        drive_id=drive_id,
                        check_name_mode=check_name_mode
                    )
                ) for j in range(ids_len)]
        ), BaseFile):
            result.append(i)

        return result



"""..."""
from dataclasses import dataclass

from .Type import DatClass


@dataclass
class VideoMediaVideoStream(DatClass):
    """..."""
    code_name: str = None
    fps: str = None
    bitrate: str = None
    clarity: str = None
    duration: str = None



"""..."""

from .AimSearchRequest import AimSearchRequest
from .AlbumListFilesRequest import AlbumListFilesRequest
from .AlbumListRequest import AlbumListRequest
from .ArchiveStatusRequest import ArchiveStatusRequest
from .ArchiveUncompressRequest import ArchiveUncompressRequest
from .BatchCancelShareRequest import BatchCancelShareRequest
from .BatchCopyFilesRequest import BatchCopyFilesRequest
from .BatchDownloadUrlRequest import BatchDownloadUrlRequest
from .BatchGetFileRequest import BatchGetFileRequest
from .BatchMoveFilesRequest import BatchMoveFilesRequest
from .BatchMoveToTrashRequest import BatchMoveToTrashRequest
from .BatchRequest import BatchRequest
from .BatchRestoreRequest import BatchRestoreRequest
from .BatchShareFileSaveToDriveRequest import BatchShareFileSaveToDriveRequest
from .BatchStarFilesRequest import BatchStarFilesRequest
from .BatchSubRequest import BatchSubRequest
from .CancelShareLinkRequest import CancelShareLinkRequest
from .CompleteFileRequest import CompleteFileRequest
from .CopyFileRequest import CopyFileRequest
from .CreateFileRequest import CreateFileRequest
from .CreateFolderRequest import CreateFolderRequest
from .CreateShareLinkRequest import CreateShareLinkRequest
from .GetAudioPlayInfoRequest import GetAudioPlayInfoRequest
from .GetDefaultDriveRequest import GetDefaultDriveRequest
from .GetDownloadUrlRequest import GetDownloadUrlRequest
from .GetDriveRequest import GetDriveRequest
from .GetFileListRequest import GetFileListRequest
from .GetFilePathRequest import GetFilePathRequest
from .GetFileRequest import GetFileRequest
from .GetRecycleBinListRequest import GetRecycleBinListRequest
from .GetShareFileListRequest import GetShareFileListRequest
from .GetShareFileRequest import GetShareFileRequest
from .GetShareInfoRequest import GetShareInfoRequest
from .GetShareLinkDownloadUrlRequest import GetShareLinkDownloadUrlRequest
from .GetShareLinkListRequest import GetShareLinkListRequest
from .GetShareTokenRequest import GetShareTokenRequest
from .GetStarredListRequest import GetStarredListRequest
from .GetUploadUrlRequest import GetUploadUrlRequest
from .GetVideoPlayInfoRequest import GetVideoPlayInfoRequest
from .GetVideoPreviewPlayInfoRequest import GetVideoPreviewPlayInfoRequest
from .ListToCleanRequest import ListToCleanRequest
from .MoveFileRequest import MoveFileRequest
from .MoveFileToTrashRequest import MoveFileToTrashRequest
from .PrivateShareRequest import PrivateShareRequest
from .RenameFileRequest import RenameFileRequest
from .RestoreFileRequest import RestoreFileRequest
from .SearchFileRequest import SearchFileRequest
from .SearchShareFileRequest import SearchShareFileRequest
from .ShareFileSaveToDriveRequest import ShareFileSaveToDriveRequest
from .StarredFileRequest import StarredFileRequest
from .TemplateRequest import TemplateRequest
from .UpdateFileRequest import UpdateFileRequest
from .UpdateShareLinkRequest import UpdateShareLinkRequest



"""Other"""
from dataclasses import dataclass, field
from typing import List

from aligo.request import GetFilePathRequest
from aligo.response import GetFilePathResponse
from aligo.types.Type import DatClass
from .BaseAligo import BaseAligo
from .Config import ADRIVE_V1_FILE_GET_PATH, ADRIVE_V1_SYSTEM_CONFIG


@dataclass
class SizeConditions(DatClass):
    blacklist: List[str] = field(default_factory=list)
    field: str = None
    max_size: int = None


@dataclass
class Share(DatClass):
    album_allowed: bool = None
    create_share_file_id_list_max_size: int = None
    extension_mode: str = None
    extension_whitelist: List[str] = field(default_factory=list)
    folder_allowed: bool = None
    mime_type_whitelist: List[str] = field(default_factory=list)
    photo_collection_allowed: bool = None
    size_conditions: List[SizeConditions] = field(default_factory=list)


@dataclass
class SystemConfig(DatClass):
    restrict_registration: bool = None
    share: Share = None


class Other(BaseAligo):
    """Other"""

    def _core_get_path(self, body: GetFilePathRequest) -> GetFilePathResponse:
        """get_path 获取当前文件的路径, 父级目录"""
        response = self.post(ADRIVE_V1_FILE_GET_PATH, body=body)
        return self._result(response, GetFilePathResponse)

    def get_system_config(self) -> SystemConfig:
        response = self.post(ADRIVE_V1_SYSTEM_CONFIG)
        return self._result(response, SystemConfig)



"""..."""
from aligo.core import BaseAligo
from aligo.core.Config import V2_DATABOX_GET_AUDIO_PLAY_INFO
from aligo.request import GetAudioPlayInfoRequest
from aligo.response import GetAudioPlayInfoResponse


class Audio(BaseAligo):
    """..."""

    def _core_get_audio_play_info(self, body: GetAudioPlayInfoRequest) -> GetAudioPlayInfoResponse:
        """获取音频播放信息"""
        response = self.post(V2_DATABOX_GET_AUDIO_PLAY_INFO, body=body)
        return self._result(response, GetAudioPlayInfoResponse)



"""..."""
from dataclasses import dataclass

from .Type import DatClass


@dataclass
class SystemTag(DatClass):
    """..."""
    name: str = None
    en_name: str = None
    confidence: int = None
    parent_en_name: str = None
    parent_name: str = None
    selected: bool = None
    tag_level: int = None
    centric_score: int = None



"""基本分享文件"""
from dataclasses import dataclass
from dataclasses import field
from typing import List

from .Enum import BaseFileCategory, BaseFileType
from .ImageMedia import ImageMedia
from .Type import DatClass
from .VideoMedia import VideoMedia
from .VideoPreview import VideoPreview


@dataclass
class BaseShareFile(DatClass):
    """..."""
    share_id: str = None
    name: str = None
    size: int = None
    creator: str = None
    description: str = None
    category: BaseFileCategory = None
    download_url: str = None
    file_extension: str = None
    file_id: str = None
    thumbnail: str = None
    type: BaseFileType = None
    updated_at: str = None
    created_at: str = None
    url: str = None
    parent_file_id: str = None
    selected: bool = None
    image_media_metadata: ImageMedia = None
    video_media_metadata: VideoMedia = None
    video_preview_metadata: VideoPreview = None
    # 2021年07月29日16时15分41秒
    mime_extension: str = None
    mime_type: str = None
    punish_flag: int = 0
    # 2021年08月31日17时56分14秒
    action_list: List[str] = field(default_factory=list, repr=False)
    # 2022年01月27日11时17分54秒
    drive_id: str = None
    domain_id: str = None
    revision_id: str = None
    # 2022年11月21日13时25分58秒
    starred: bool = False
    content_hash: str = None
    trashed_at: str = None
    from_share_id: str = None



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DatClass


@dataclass
class BatchCopyFilesRequest(DatClass):
    """..."""
    drive_id: str = None
    file_id_list: List[str] = field(default_factory=list)
    to_parent_file_id: str = 'root'
    overwrite: bool = False
    auto_rename: bool = True



"""..."""
from dataclasses import dataclass
from dataclasses import field
from typing import Dict, List

from .Enum import BaseFileType, BaseFileCategory, BaseFileContentHashName
from .FieldsInfo import FieldsInfo
from .ImageMedia import ImageMedia
from .Type import DatClass
from .VideoMedia import VideoMedia
from .VideoPreview import VideoPreview


@dataclass
class BaseFile(DatClass):
    """..."""
    type: BaseFileType = None
    file_id: str = None
    name: str = None
    parent_file_id: str = False
    category: BaseFileCategory = field(default=None, repr=False)
    size: int = field(default=None, repr=False)
    created_at: str = field(default=None, repr=False)
    content_type: str = field(default=None, repr=False)
    description: str = field(default=None, repr=False)
    content_hash: str = field(default=None, repr=False)
    content_hash_name: BaseFileContentHashName = field(default=None, repr=False)
    crc64_hash: str = field(default=None, repr=False)
    domain_id: str = field(default=None, repr=False)
    download_url: str = field(default=None, repr=False)
    drive_id: str = field(default=None, repr=False)
    encrypt_mode: str = field(default=None, repr=False)
    file_extension: str = field(default=None, repr=False)
    hidden: bool = field(default=None, repr=False)
    image_media_metadata: ImageMedia = field(default=None, repr=False)
    labels: list = field(default=None, repr=False)
    meta: str = field(default=None, repr=False)
    mime_extension: str = field(default=None, repr=False)
    mime_type: str = field(default=None, repr=False)
    punish_flag: int = field(default=None, repr=False)
    starred: bool = field(default=None, repr=False)
    status: str = field(default=None, repr=False)
    streams_url_info: Dict = field(default=None, repr=False)
    streams_info: Dict = field(default=None, repr=False)
    thumbnail: str = field(default=None, repr=False)
    trashed: bool = field(default=None, repr=False)
    trashed_at: str = field(default=None, repr=False)
    updated_at: str = field(default=None, repr=False)
    upload_id: str = field(default=None, repr=False)
    url: str = field(default=None, repr=False)
    user_meta: str = field(default=None, repr=False)
    video_media_metadata: VideoMedia = field(default=None, repr=False)
    video_preview_metadata: VideoPreview = field(default=None, repr=False)
    location: str = field(default=None, repr=False)
    action_list: List[str] = field(default_factory=list, repr=False)
    user_tags: Dict = field(default=None, repr=False)
    last_modifier_type: str = field(default=None, repr=False)
    last_modifier_id: str = field(default=None, repr=False)
    last_modifier_name: str = field(default=None, repr=False)
    creator_type: str = field(default=None, repr=False)
    creator_id: str = field(default=None, repr=False)
    creator_name: str = field(default=None, repr=False)
    revision_id: str = field(default=None, repr=False)
    sync_flag: bool = field(default=None, repr=False)
    sync_device_flag: bool = field(default=None, repr=False)
    sync_meta: str = field(default=None, repr=False)
    ex_fields_info: FieldsInfo = field(default=None, repr=False)
    from_share_id: str = field(default=None, repr=False)
    revision_version: int = field(default=None, repr=False)
    channel: str = field(default=None, repr=False)
    meta_name_punish_flag: int = field(default=None, repr=False)
    meta_name_investigation_status: int = field(default=None, repr=False)



"""..."""
from dataclasses import dataclass

from .Type import DatClass


@dataclass
class ImageQuality(DatClass):
    """..."""
    clarity: int = None
    clarity_score: int = None
    color: int = None
    color_score: int = None
    composition_score: int = None
    contrast: int = None
    contrast_score: int = None
    exposure: int = None
    exposure_score: int = None
    overall_score: int = None



"""..."""
from dataclasses import dataclass
from typing import List

from aligo.types import DatClass, VideoTranscodeTemplate


@dataclass
class GetVideoPlayInfoResponse(DatClass):
    """..."""
    template_list: List[VideoTranscodeTemplate] = None



"""..."""
from typing import Iterator

from aligo.core import BaseAligo
from aligo.core.Config import V2_FILE_MOVE
from aligo.request import BatchMoveFilesRequest, MoveFileRequest, BatchRequest, BatchSubRequest
from aligo.response import BatchSubResponse, MoveFileResponse


class Move(BaseAligo):
    """..."""

    def _core_move_file(self, body: MoveFileRequest) -> MoveFileResponse:
        """..."""
        response = self.post(V2_FILE_MOVE, body=body)
        return self._result(response, MoveFileResponse)

    def _core_batch_move_files(self, body: BatchMoveFilesRequest) -> Iterator[BatchSubResponse[MoveFileResponse]]:
        """..."""
        if body.drive_id is None:
            body.drive_id = self.default_drive_id

        yield from self.batch_request(BatchRequest(
            requests=[BatchSubRequest(
                id=file_id,
                url='/file/move',
                body=MoveFileRequest(
                    drive_id=body.drive_id, file_id=file_id,
                    to_parent_file_id=body.to_parent_file_id,
                    overwrite=body.overwrite, auto_rename=body.auto_rename
                )
            ) for file_id in body.file_id_list]
        ), MoveFileResponse)



"""..."""
from dataclasses import dataclass, field
from typing import List, Dict

from aligo.types import DatClass, UploadPartInfo
from aligo.types.Enum import BaseFileType, BaseFileStatus


@dataclass
class CreateFileResponse(DatClass):
    """..."""
    file_name: str = None
    type: BaseFileType = None
    file_id: str = None
    parent_file_id: str = None
    domain_id: str = field(default=None, repr=False)
    drive_id: str = field(default=None, repr=False)
    encrypt_mode: str = field(default=None, repr=False)
    part_info_list: List[UploadPartInfo] = field(default_factory=list, repr=False)
    rapid_upload: bool = field(default=None, repr=False)
    status: BaseFileStatus = field(default=None, repr=False)
    streams_upload_info: Dict = field(default=None, repr=False)
    upload_id: str = field(default=None, repr=False)
    exist: bool = field(default=None, repr=False)
    location: str = field(default=None, repr=False)
    pre_hash: str = field(default=None, repr=False)  # pre_hash
    code: str = field(default=None, repr=False)  # 与批量操作中的错误不同, 此处code为是否命中秒传的结果
    message: str = field(default=None, repr=False)
    revision_id: str = field(default=None, repr=False)



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types.Enum import TemplateType
from aligo.types import DatClass


@dataclass
class TemplateResponse(DatClass):
    """..."""



"""..."""
from dataclasses import dataclass

from .Type import DatClass


@dataclass
class UploadPartInfo(DatClass):
    """
    {
        "part_number": 1
    }
    """
    etag: str = None
    part_number: int = None
    part_size: int = None
    upload_url: str = None
    internal_upload_url: str = None
    content_type: str = None
    upload_form_info: str = None
    internal_upload_form_info: str = None



"""..."""
from dataclasses import dataclass
from typing import List

from .ImageTag import ImageTag
from .Type import DatClass
from .VideoMediaAudioStream import VideoMediaAudioStream
from .VideoMediaVideoStream import VideoMediaVideoStream


@dataclass
class VideoMedia(DatClass):
    """..."""
    time: str = None
    city: str = None
    country: str = None
    address_line: str = None
    district: str = None
    duration: str = None
    height: int = None
    location: str = None
    province: str = None
    township: str = None
    video_media_audio_stream: List[VideoMediaAudioStream] = None
    video_media_video_stream: List[VideoMediaVideoStream] = None
    width: int = None
    image_tags: List[ImageTag] = None



"""..."""
from dataclasses import dataclass

from .Type import DatClass


@dataclass
class AudioMusicMeta(DatClass):
    """..."""
    title: str = None
    artist: str = None
    album: str = None
    cover_url: str = None



"""..."""
from dataclasses import dataclass

from aligo.types import DatClass
from aligo.types.Enum import BaseFileType, BaseFileCategory


@dataclass
class ShareItemInfo(DatClass):
    """..."""
    category: BaseFileCategory = None
    file_extension: str = None
    file_id: str = None
    file_name: str = None
    thumbnail: str = None
    type: BaseFileType = None



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DatClass, BaseFile


@dataclass
class ListResponse(DatClass):
    """..."""
    items: List[BaseFile] = field(default_factory=list)
    next_marker: str = ''
    punished_file_count: int = 0



"""更新分享链接响应"""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DatClass
from aligo.types.Enum import SharePolicy
from aligo.types.ShareLinkBaseFile import ShareLinkBaseFile


@dataclass
class UpdateShareLinkResponse(DatClass):
    """更新分享链接响应"""
    share_id: str
    share_name: str = None
    share_pwd: str = None
    expiration: str = None
    created_at: str = field(default=None, repr=False)
    updated_at: str = field(default=None, repr=False)
    creator: str = field(default=None, repr=False)
    description: str = field(default=None, repr=False)
    download_count: int = field(default=0, repr=False)
    preview_count: int = field(default=0, repr=False)
    save_count: int = field(default=0, repr=False)
    drive_id: str = field(default=None, repr=False)
    expired: str = field(default=None, repr=False)
    file_id: str = field(default=None, repr=False)
    file_id_list: List[str] = field(default=None, repr=False)
    share_msg: str = field(default=None, repr=False)
    share_policy: SharePolicy = field(default=None, repr=False)
    share_url: str = field(default=None, repr=False)
    status: str = field(default=None, repr=False)
    first_file: ShareLinkBaseFile = field(default=None, repr=False)
    is_subscribed: bool = field(default=False, repr=False)
    num_of_subscribers: int = field(default=0, repr=False)
    display_name: str = field(default=None)
    current_sync_status: int = field(default=None, repr=False)
    next_sync_status: int = field(default=None, repr=False)
    full_share_msg: str = field(default=None, repr=False)
    ex_status: int = field(default=None, repr=False)
    popularity: int = None
    popularity_str: str = None
    file_path_list: List[str] = field(default_factory=list)
    drive_file_list: List[str] = None
    report_count: int = 0
    video_preview_count: int = 0
    category: str = None
    enable_file_changed_notify: bool = False
    disable_preview: bool = False
    disable_save: bool = False
    disable_download: bool = False
    preview_limit: int = 0
    save_download_limit: int = 0
    require_login: bool = False
    enable_upload: bool = False



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DatClass, DriveFile


@dataclass
class PrivateShareResponse(DatClass):
    drive_file_list: List[DriveFile] = field(default_factory=list)
    expiration: str = None
    expired: bool = None
    full_share_msg: str = None
    share_id: str = None
    share_name: str = None
    share_subtitle: str = None
    share_title: str = None
    share_url: str = None
    thumbnail: str = None



"""获取分享文件列表"""
from dataclasses import dataclass, field

from aligo.types import DatClass
from aligo.types.Enum import BaseFileCategory, BaseFileType, GetShareFileListOrderBy, OrderDirection


@dataclass
class GetShareFileListRequest(DatClass):
    """..."""
    share_id: str = None
    starred: bool = None
    all: bool = None
    category: BaseFileCategory = None
    fields: str = None
    image_thumbnail_process: str = None
    image_url_process: str = field(default='image/resize,w_1920/format,jpeg', repr=False)
    limit: int = None
    marker: str = None
    order_by: GetShareFileListOrderBy = 'name'
    order_direction: OrderDirection = 'DESC'
    parent_file_id: str = 'root'
    status: str = None
    type: BaseFileType = None
    url_expire_sec: int = field(default=14400, repr=False)
    video_thumbnail_process: str = field(default='video/snapshot,t_0,f_jpg,ar_auto,w_800', repr=False)



"""..."""
import requests


class Null:
    """表示一个失败的结果"""

    def __init__(self, response: requests.Response):
        self.response = response
        # noinspection PyBroadException
        try:
            d = response.json()
        except Exception:
            pass
        else:
            self.code = d.get('code')
            self.message = d.get('message')
            self.requestId = d.get('requestId')
            self.resultCode = d.get('resultCode')

    def __repr__(self):
        return self.response.text

    def __bool__(self):
        return False



"""..."""
from dataclasses import dataclass

from aligo.types import DatClass


@dataclass
class ShareLinkExtractCodeResponse(DatClass):
    """..."""
    share_id: str = None
    share_pwd: str = None



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DatClass, ListAlbumItem


@dataclass
class AlbumListResponse(DatClass):
    """..."""
    items: List[ListAlbumItem] = field(default_factory=list, repr=False)
    next_marker: str = ''
    album_count: int = None



"""..."""
import warnings
from typing import List, overload

from aligo.core import Core
from aligo.request import SearchFileRequest, AimSearchRequest
from aligo.types import BaseFile
from aligo.types.Enum import SearchCategory, BaseFileCategory


class Search(Core):
    """搜索相关"""

    def search_file(self, *args, **kwargs):
        """请使用 search_files """
        warnings.warn('此方法已更名为 `search_files`', DeprecationWarning, stacklevel=2)
        return self.search_files(*args, **kwargs)

    @overload
    def search_files(self, name: str = None, category: SearchCategory = None,
                     drive_id: str = None, **kwargs) -> List[BaseFile]:
        """
        搜索文件
        :param name: [必选] 搜索的文件名
        :param category: [可选] 搜索的文件类型
        :param drive_id: [可选] 搜索的文件所在的网盘
        :param kwargs: [可选] 其他参数
        :return: [List[BaseFile]]

        用法示例：
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> files1 = ali.search_files('test')
        >>> files2 = ali.search_files('test', category='video')
        """

    @overload
    def search_files(self, body: SearchFileRequest) -> List[BaseFile]:
        """
        搜索文件
        :param body: [必选] 搜索文件请求对象
        :return: [List[BaseFile]]

        用法示例：
        >>> from aligo import Aligo, SearchFileRequest
        >>> ali = Aligo()
        >>> files = ali.search_files(body=SearchFileRequest(query='name match "test"'))
        >>> print(files)
        """

    def search_files(self, name: str = None, category: SearchCategory = None, parent_file_id: str = 'root',
                     drive_id: str = None, body: SearchFileRequest = None, **kwargs) -> List[BaseFile]:
        """search files"""
        if body is None:
            query = ''
            if parent_file_id != 'root':
                query = f'parent_file_id = "{parent_file_id}"'
            if name:
                if query:
                    query += ' and '
                query += f'name match "{name}"'
            if category is not None:
                if query:
                    query += ' and '
                query += f'category = "{category}"'
            body = SearchFileRequest(query=query, drive_id=drive_id, **kwargs)
        result = self._core_search_files(body)
        return list(result)

    @overload
    def search_aims(self, keyword: str, category: BaseFileCategory = 'image',
                    drive_id: str = None, **kwargs) -> List[BaseFile]:
        """
        搜索目标/标签
        :param keyword: [必选] 搜索的关键字
        :param category: [可选] 搜索的文件类型
        :param drive_id: [可选] 搜索的文件所在的网盘
        :param kwargs: [可选] 其他参数
        :return: [List[BaseFile]]

        用法示例：
        >>> from aligo import Aligo
        >>> ali = Aligo()
        >>> files1 = ali.search_aims('test')
        >>> files2 = ali.search_aims('test', category='video')
        """

    @overload
    def search_aims(self, body: AimSearchRequest) -> List[BaseFile]:
        """
        搜索目标/标签
        :param body: [必选] 搜索目标/标签请求对象
        :return: [List[BaseFile]]

        用法示例：
        >>> from aligo import Aligo, AimSearchRequest
        >>> ali = Aligo()
        >>> files = ali.search_aims(body=AimSearchRequest(query='keywords = "test"'))
        >>> print(files)
        """

    def search_aims(self, keyword: str = None, category: BaseFileCategory = 'image', drive_id: str = None,
                    body: AimSearchRequest = None, **kwargs) -> List[BaseFile]:
        """search_aims"""
        if body is None:
            body = AimSearchRequest(
                query=f"keywords = '{keyword}' and type = 'file' and category = '{category}'",
                drive_id=drive_id,
                **kwargs
            )
        result = self._core_search_aims(body)
        return list(result)



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DatClass
from aligo.types.Enum import TemplateType


@dataclass
class TemplateRequest(DatClass):
    """..."""



"""..."""
from dataclasses import dataclass

from aligo.types import DatClass
from .GetDownloadUrlResponse import GetDownloadUrlResponse


@dataclass
class BatchDownloadUrlResponse(DatClass):
    """..."""
    id: str = None
    status: int = None
    body: GetDownloadUrlResponse = None
    method: str = None
